package com.urbancode.air.plugin.servicecontrolmanager;

import com.urbancode.air.CommandHelper;
import org.apache.commons.io.output.NullOutputStream;

public class ServiceControlManagerHelper {

    def service;
    def waitFor;
    def argStrings;
    def ch;

    public ServiceControlManagerHelper(def props) {
        service = props['Service'];
        waitFor = Boolean.valueOf(props['waitFor']);
        argStrings = (props['argString']?props['argString'].split('\n'):null);
        ch = new CommandHelper(new File('.'));
        ch.ignoreExitValue=true;
    }

    public void startService() {
        def args = ['sc.exe', 'start', service];
        argStrings.each { arg ->
            args << arg;
        }
        def exitCode = runCommand("Starting Service $service", args);

        if (exitCode != 0) {
            println "Exit Code: " + exitCode;
            System.exit 1
        }
        else if (waitFor) {
            waitForServiceToBeStarted()
        }
    }

    public void stopService() {
        def args = ['sc.exe', 'stop', service];
        argStrings.each { arg ->
            args << arg;
        }
        def exitCode = runCommand("Stoping Service $service", args);

        if (exitCode != 0) {
            println "Exit Code: " + exitCode;
            System.exit 1
        }
        else if (waitFor) {
            waitForServiceToBeStopped();
        }
    }

    public void createService() {
        def args = ['sc.exe', 'create', service];
        argStrings.each { arg ->
             args << arg;
        }
        def exitCode = runCommand("Creating service $service", args);

        if (exitCode == 0) {
            System.exit 0
        }
        else { 
            println "Exit Code: " + exitCode;
            System.exit 1
        }
    }

    public void deleteService() {
        def args = ['sc.exe', 'delete', service];
        argStrings.each { arg ->
             args << arg;
        }
        def exitCode = runCommand("Deleting service $service", args);

        if (exitCode == 0) {
            System.exit 0
        }
        else {
            println "Exit Code: " + exitCode;
            System.exit 1
        }
    }

    public void checkIfExists() {
        def args = ['sc.exe', 'query', service];
        def exitCode = runCommand("Checking for service $service", args);

        if (exitCode == 0) {
            System.exit 0
        }
        else {
            println "Exit Code: " + exitCode;
            System.exit 1
        }
    }

    public void waitForServiceToBeStarted() {
        println "Waiting for service to start";
        def started = false;
        def args = ['sc.exe', 'query', service];
        def nullStream = new NullOutputStream();
        def nullPrintStream = new PrintStream(nullStream);
        def curSysOut = System.out;
        System.setOut(nullPrintStream);
        while (!started) {
            runCommand("Checking Service Status", args) { output ->
               if (output.contains("RUNNING")) {
                   started = true;
               }
               else if (output.contains("STOPPED")) {
                   throw new Exception("Service moved back to stopped state. Check Service log for failures.");
               }
           }
           Thread.sleep(1000);
       }
       System.setOut(curSysOut);
    }

    public void waitForServiceToBeStopped() {
        println "Waiting for service to stop";
        def stopped = false;
        def args = ['sc.exe', 'query', service];
        def nullStream = new NullOutputStream();
        def nullPrintStream = new PrintStream(nullStream);
        def curSysOut = System.out;
        System.setOut(nullPrintStream);
        while (!stopped) {
            runCommand("Checking Service Status", args) { output ->
               if (output.contains("STOPPED")) {
                   stopped = true;
               }
               else if (!output.contains("STOP_PENDING")) {
                   throw new Exception("Service not in STOP PENDING state. Check Service log for failures.");
               }
           }
           Thread.sleep(1000);
       }
       System.setOut(curSysOut);
    }


    private int runCommand(def message, def args) {
       runCommand(message, args, null);
    }

    private int runCommand(def message, def args, def closure) {
        def result;
        def outStream = new ByteArrayOutputStream();
        def exitCode = ch.runCommand(message, args) { proc ->

            proc.out.close();//close std in
            proc.waitForProcessOutput(outStream, outStream);
            result = outStream.toString();
        }

        if (result.contains("FAILED") || result.contains("SYNTAX") || result.contains("USAGE")) {
            throw new Exception("Command Failed:\n" + result);
        }
        System.out.println("Command Result : ");
        System.out.println(result);

        if (closure) {
            closure(result);
        }

        return exitCode
    }
}
